#!/usr/bin/env python2
# -*- coding: utf-8 -*-

# import ../inAndOut/Escrita
import datetime
from ..exceptions.InconsistenciaException import InconsistenciaException
# O arquivo ../exceptions/InconsistenciaException.py é um módulo de nome igual ao nome do arquivo
# A linha acima foi utilizada para importar apenas a classe InconsistenciaException desse módulo.

class DadosDeEntrada():
    #Esta classe "agrupa" os objetos lidos através de planilhas (leitura é feita através de um objeto
    #da classe Leitura). Para facilitar o tratamento de exceções, Mapas são utilizados para armazenar docentes
    #(a chave utilizada é o código deste docentes), e veículos (a chave utilizada é a sigla deste veículo).
    #Para facilitar a geração de relatórios (onde docentes devem estar ordenados por nome em ordem crescente)
    #um segundo mapa de docentes é utilizado (neste, a chave é o nome do docente);
    def __init__(self):
        self.mapaDocentesNome = {}
        self.mapaDocentesCodigo = {}
        self.listaPublicacoes = []
        self.listaRegras = []
        self.mapaVeiculos = {}
        self.ano = None

    #Adiciona um docente aos dois mapas de docente (chaves Nome e Código)
    def adicionaDocente(self, docente):
        #Caso não existe nenhum docente com o nome (e código) do novo docente,
        #o novo é adicionado aos mapa. Se já existe, o novo docente não é adicionado,
        #mas é apenas deletado
        if docente.nome not in self.mapaDocentesNome and docente.codigo not in self.mapaDocentesCodigo:
            self.mapaDocentesNome[docente.nome] = docente
            self.mapaDocentesCodigo[docente.codigo] = docente


    #Adiciona uma publicação a lista encadeada de publicações
    def adicionaPublicacao(self, publicacao):
        self.listaPublicacoes.append(publicacao)

    #Adiciona uma regra a lista encadeada de regras
    def adicionaRegra(self, regra):
        self.listaRegras.append(regra)

    #Adiciona um veiculo ao mapa de veículos (a chave é a sigla do veículo)
    def adicionaVeiculo(self, veiculo):
        #Adiciona um veículo somente se ele não existir no mapa de veículos.
        if veiculo.sigla not in self.mapaVeiculos:
            self.mapaVeiculos[veiculo.sigla] = veiculo

    #Confere se o código do docente já está cadastrado
    def codigoDocenteExiste(self, codigo):
        return codigo in self.mapaDocentesCodigo

    #retorna um docente de acordo com o código solicitado
    def getDocenteCodigo(self,codigo):
        # Se o docente com o codigo informado existe, retorna uma referencia para ele;
        # Caso não exista, retorna nulo;
        if codigo in self.mapaDocentesCodigo:
            return self.mapaDocentesCodigo[codigo]
        return None

    #retorna um docente que tem o nome solicitado
    def getDocenteNome(self, nome):
        # Se o docente com o nome informado existe, retorna uma referencia para ele;
        # Caso não exista, retorna nulo;
        if nome in self.mapaDocentesNome:
            return self.mapaDocentesNome[nome]
        return None

    #Confere se a sigla de veículo já está cadastrada
    def siglaVeiculoExiste(self, sigla):
        return sigla in self.mapaVeiculos

    #Retorna o veículo com a sigla solicitada
    def getVeiculo(self, sigla):
        # Se o veiculo com a sigla informada existe, retorna uma referencia para ele;
        # Caso não exista, retorna nulo;
        if sigla in self.mapaVeiculos:
            return self.mapaVeiculos[sigla]
        return None

    #Função que retorna uma lista de publicações cujo veículo associado tem o qualis solicitado
    def getListaPublicacoesQualis(self, qualis):
        # O qualis de um artigo é o vigente em sua data de publicação, e não o atual.
        # (Conferido com o prof. Vitor)
        # Assim, as publicacoes cujo qualis em sua data de publicacao eh igual ao
        # qualis passado como parâmetro são adicionadas a lista, que é retornada
        listaPubQualis = []
        for publicacao in self.listaPublicacoes:
            if publicacao.getQualificacaoVeiculoNoAno(publicacao.ano) == qualis:
                listaPubQualis.append(publicacao)
        return listaPubQualis


    #Acha a regra vigente na data atual
    def getRegraVigente(self, data):
        #A data desejada deve ser maior ou igual a data de inicio de vigência da regra
        for regra in self.listaRegras:
            inicioVig = regra.inicioVigencia
            fimVig = regra.fimVigencia
            if data >= inicioVig and data <= fimVig:
                return regra
        return None


    #função que recebe uma string e "converte" para a variável ano (inteira)
    #essa função funciona como uma "Setter" que converte String para inteiro;
    def parseAno(self, anoString):
        #se int() falha, ele lança uma exceção do tipo invalid_argument.
        #Se o ano for passado de forma errada, uma exceção é lançada.
        try:
            self.ano = int(anoString)
        except ValueError:
            raise InconsistenciaException("Erro! Digite um ano para ser analisado!")
