#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import Publicacao
import Regra
import datetime

class Docente():
    """docstring for Docente"""
    #Esta classe representa um docente cadastrado no sistema, armazenando seus respectivos dados.

    #Construtor da Classe
    def __init__(self, codigo = None, nome = None, dataNascimento = None, dataIngresso = None,\
                 ehCoordenador = False):
        self.codigo = codigo
        self.nome = nome
        self.dataNascimento = dataNascimento
        self.dataIngresso = dataIngresso
        self.listaPublicacoes = []
        self.ehCoordenador = ehCoordenador

    #Não existem atributos privados em python, portanto não são necessários os getters/setters

    #adiciona uma publicação a lista de publicações do docente
    def adicionaPublicacao(self, publicacao):
        self.listaPublicacoes.append(publicacao)

    #Retorna a idade do Docente
    def getIdade(self, dataAtual):
        idade = dataAtual.year - self.dataNascimento.year
        diferencaMes = dataAtual.month - self.dataNascimento.month
        if diferencaMes < 0:
            idade = idade - 1
        elif diferencaMes == 0:
            diferencaDias = dataAtual.day - self.dataNascimento.day
            if (diferencaDias < 0):
                idade = idade - 1

        return idade

    #calcula a pontuação de um docente a partir de uma determinada regra;
    #e retorna o valor da pontuação.
    def calculaPontuacao(self, anoVigencia, regra):
        #Nesse ponto é considerado que a regra existe (caso contrário já houve tratamento de exceção)
        #E que a data fornecida é válida (01/01/ANO_VIGENCIA)
        pontuacao = 0.0
        for publicacao in self.listaPublicacoes:
            #Encontra a data mínima (ano - número de anos para considerar da regra)
            #ex: analisando 2 anos anteriores a regra de 2017, os artigos válidos são aqueles de 2015 e 2016;

            inicioVigencia = datetime.datetime.strptime("01 01 {}".format(anoVigencia.year - regra.anosparaConsiderar),"%d %m %Y");

            #Se a data do artigo NÃO for ANTES do inicio da vigencia E for ANTES da data atual, ele pontua

            #se dataPublicacao >= inicioVigencia, significa que o artigo foi publicado depois
            #(ou na data) do inicio da vigencia.

            #Se a diferença dataAtual (ano) - dataPublicacao for > 0, significa que a data atual ocorre
            #depois da data da publicação (ou que o artigo foi publicado antes da data atual)
            #Equivalente a dizer que dataPublicacao < dataAtual(ano)

            if (publicacao.ano >= inicioVigencia.year and publicacao.ano < anoVigencia.year):
                multiplicador = regra.multiplicadorPeriodico if publicacao.getClass() == "PublicacaoPeriodico" else 1.0

                # A pontuação de um artigo é obtida a partir da regra, que tem um método para retornar a pontuação
                # de um determinado qualis (String). O qualis é obtido a partir de publicação, que acessa o veículo
                # em que ela foi publicada. O veículo, dado um ano, retorna a qualificação (String) vigente nesta data.

                # O qualis de um artigo é o de sua data de publicação, e não o atual. (Conferido com o prof.)
                # Dado uma publicação, vai em seu veículo e pega a qualificação vigente no ano da publicação
                # Com o qualis da publicação (String), pega a pontuação correspondente na regra vigente
                qualif = publicacao.getQualificacaoVeiculoNoAno(publicacao.ano)
                if (qualif != "NC"):
                    pontuacao = pontuacao + multiplicador*(regra.getPontuacaoQualis(qualif))

        return pontuacao
