#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import Docente
import Veiculo
from abc import ABCMeta, abstractmethod #Modulo para classes abstratas

class Publicacao:
    """docstring for Publicacao"""
    __metaclass__ = ABCMeta
    # Esta classe representa uma publicação cadastrada no sistema, armazenando seus respectivos dados.
    # Esta classe é abstrata, e portanto, não podem existir objetos dela.

    def __init__(self, ano = None, numero = None, pagInicial = None, pagFinal = None, titulo = None, \
                 listaDocentes = []):
        self.ano = ano
        self.numero = numero
        self.pagInicial = pagInicial
        self.pagFinal = pagFinal
        self.titulo = titulo
        self.listaDocentes = listaDocentes
        self.__veiculo = None


    def setVeiculo(self,veiculo):
        if (self.__veiculo == None):
            self.__veiculo = veiculo

    # Não existem atributos privados em python, portanto não são necessários os getters/setters
    #retorna numero de docentes (autores) de uma publicação
    def getNumeroDocentes(self):
        return len(self.listaDocentes)

    #retorna a sigla do veiculo da publicação
    def getSiglaVeiculo(self):
        #Se o veículo da publicaçao existir, retorna sua sigla
        r = None
        if self.__veiculo != None:
            r = self.__veiculo.sigla
        return r

    #retorna a qualificação do veículo desta publicação para um dado ano
    def getQualificacaoVeiculoNoAno(self, ano):
        return self.__veiculo.getQualificacaoAno(ano)

    #retorna o nome do veiculo desta publicação
    def getNomeVeiculo(self):
        return self.__veiculo.nome

    #retorna o fator de impacto do veiculo desta publicação
    def getFatorImpactoVeiculo(self):
        return self.__veiculo.fatorImpacto

    #Retorna o nome dos docentes desta publicação (separados por vírgula)
    def getNomeDocentes(self):
        str = ""
        for docente in self.listaDocentes:
            str = str + docente.nome + ","
        return str[:-1]

    #Método virtual para retornar o nome da classe (em python isso já existe: objeto.__name__)
    #Mas é necessário ter um método abstrato para que a classe seja abstrata...
    @abstractmethod
    def getClass(self):
        pass

    def returnTupleComparator(self): 
        #Ordenação com múltiplos atributos pode ser feita usando uma tupla!
        #Onde o elemento em i tem prioridade sobre o elemento em i+1...
        return (-self.ano, self.getSiglaVeiculo(), self.titulo)
