#!/usr/bin/env python2
# -*- coding: utf-8 -*-

from ..classes.Publicacao import Publicacao
from ..classes.Veiculo import Veiculo
from ..exceptions.ErroFormatacaoException import ErroFormatacaoException
from ..exceptions.InconsistenciaException import InconsistenciaException
from ..exceptions.FileNotFoundException import FileNotFoundException
import locale
from datetime import datetime
import io

locale.setlocale(locale.LC_ALL, 'pt_BR.utf8') 

class Escrita():
    #Classe responsável por escrever os arquivos (planilhas) cujos nomes são definidos na construção de
    #objetos desta classe. Os dados são extraídos de um objeto do tipo "DadosDeEntrada" e escritos
    #em novas planilhas ou serializados em um arquivo binário. Atributos para lidar com local e
    #formato de data são definidos como constantes, pois não podem ser alteradas.

    def __init__(self, catQualis, nomeArqRec, nomeArqPub, nomeArqEst):
        self.categoriasQualis = catQualis
        self.nomeArquivoRecredenciamento = nomeArqRec
        self.nomeArquivoPublicacoes = nomeArqPub
        self.nomeArquivoEstatisticas = nomeArqEst

    def __geraRelatorioPublicacoes(self, dadosEntrada):
        try:
            with io.open(self.nomeArquivoPublicacoes,"w",encoding="utf8") as arqSaida:
                arqSaida.write(u"Ano;Sigla Veículo;Veículo;Qualis;Fator de Impacto;Título;Docentes\n");
                for qualis in self.categoriasQualis:
                    #Como este relatório é ordenado por Qualis (os elementos de categoriasQualis devem estar ordenados);
                    #uma função que retorna todas as publicações de determinado qualis é chamada:
                    listaPublicacoes = dadosEntrada.getListaPublicacoesQualis(qualis)
                    listaPublicacoes = sorted(listaPublicacoes,key=Publicacao.returnTupleComparator)
                    for pub in listaPublicacoes:
                        arqSaida.write(u"{};{};{};{};{};{};{}\n".format(pub.ano,pub.getSiglaVeiculo(),\
                                                                       pub.getNomeVeiculo(),pub.getQualificacaoVeiculoNoAno(pub.ano),\
                                                                       locale.format("%.3f",pub.getFatorImpactoVeiculo()),\
                                                                       pub.titulo, pub.getNomeDocentes()))
                # Escrever as qualificações "NC", se existirem
                listaPublicacoes = dadosEntrada.getListaPublicacoesQualis("NC");
                if (len(listaPublicacoes) > 0):
                    listaPublicacoes = sorted(listaPublicacoes,key=Publicacao.returnTupleComparator);
                for pub in listaPublicacoes:
                    arqSaida.write(u"{};{};{};{};{};{};{}\n".format(pub.ano,pub.getSiglaVeiculo(),\
                                                                   pub.getNomeVeiculo(),pub.getQualificacaoVeiculoNoAno(pub.ano),\
                                                                   locale.format("%.3f",pub.getFatorImpactoVeiculo()),\
                                                                   pub.titulo, pub.getNomeDocentes()))
        except IOError:
            raise FileNotFoundException();

    def __geraRelatorioEstatisticas(self,dadosEntrada):
        try:
            with io.open(self.nomeArquivoEstatisticas,"w",encoding="utf8") as arqSaida:
                arqSaida.write(u"Qualis;Qtd. Artigos;Média Artigos / Docente\n")
                for qualis in self.categoriasQualis:
                    # Este relatório é ordenado por Qualis. (os elementos de categoriasQualis devem estar ordenados)
                    #A lista de publicações do qualis analisado é recuperada:
                    listaPublicacoes = dadosEntrada.getListaPublicacoesQualis(qualis)
                    # número de artigos por docente em cada Qualis (mesmo somatório, só que cada artigo soma 1
                    # dividido pelo número de docentes autores).
                    numArtigosDocente = 0.0
                    for pub in listaPublicacoes:
                        numArtigosDocente = numArtigosDocente + 1.0/pub.getNumeroDocentes();

                    #O número é escrito no arquivo
                    arqSaida.write(u"{};{};{}\n".format(qualis,len(listaPublicacoes),locale.format("%.2f",numArtigosDocente)))
        except IOError:
            raise FileNotFoundException()


    def __geraRelatorioRecredenciamento(self,dadosEntrada):
        try:
            with io.open(self.nomeArquivoRecredenciamento,"w",encoding="utf8") as arqSaida:
                arqSaida.write(u"Docente;Pontuação;Recredenciado?\n")
                dadosEntrada.ano = 2017
                anoCalculo = datetime.strptime(u"01 01 {}".format(dadosEntrada.ano),"%d %m %Y")
                regraVigente = dadosEntrada.getRegraVigente(anoCalculo);
                if regraVigente == None:
                    raise InconsistenciaException(u"Não há regra vigente na data informada");
                for nomeDocente in sorted(dadosEntrada.mapaDocentesNome):
                    #Para cada nome de docente no mapa de docentes, recupera o objeto referente a este docente:
                    docente = dadosEntrada.mapaDocentesNome[nomeDocente]
                    #chama um método para calcular a pontuação do docente
                    pontuacaoDocente = docente.calculaPontuacao(anoCalculo,regraVigente)
                    recredenciado = ""
                    #Define a data limite para que um docente seja "PPJ"
                    dataNovato = docente.dataIngresso
                    dataNovato = datetime.strptime("{} {} {}".format(dataNovato.day,dataNovato.month,dataNovato.year+3),"%d %m %Y")
                    #Calcula a idade do docente (em anos, por isso deve ser usado o >= 60)
                    idadeDocente = docente.getIdade(anoCalculo)

                    #Define o credenciamento do docente de acordo com a categoria:
                    #Se ele é coordenador, novo no programa, tem mais de 60 anos ou
                    #superou a pontuação mínima definida na regra. Caso ele não atenda
                    #nenhum dos requisitos acima, ele não é recredenciado.
                    if docente.ehCoordenador:
                        recredenciado = u"Coordenador"
                    elif dataNovato >= anoCalculo:
                        recredenciado = u"PPJ"
                    elif idadeDocente >= 60:
                        recredenciado = u"PPS";
                    elif pontuacaoDocente >= regraVigente.pontuacaoMinima:
                        recredenciado = u"Sim"
                    else:
                        recredenciado = u"Não"
                    arqSaida.write(u"{};{};{}\n".format(docente.nome,locale.format("%.1f",pontuacaoDocente),recredenciado));
        except IOError:
            raise FileNotFoundException()
        

    def escreveRelatorios(self,dadosEntrada):
        self.__geraRelatorioPublicacoes(dadosEntrada)
        self.__geraRelatorioRecredenciamento(dadosEntrada)
        self.__geraRelatorioEstatisticas(dadosEntrada)

